library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.std_logic_unsigned.all;
 
entity test is
end test;
 
architecture test of test is
 
	component main
		port(
		clockIn : in std_logic;
		data :  in std_logic_vector(1 downto 0);
		controlData : in std_logic;
		controlClock : in std_logic;
		outEnable : in std_logic;
		rawClockOut : out std_logic;
		out1 : out std_logic;
		out2 : out std_logic;
		clockOut : out std_logic
		);
	end component;


	--Inputs
	signal clockIn : std_logic := '0';
	signal data : std_logic_vector(1 downto 0) := (others => '0');
	signal controlData : std_logic := '0';
	signal controlClock : std_logic := '0';
	signal outEnable : std_logic := '1';

	--Outputs
	signal rawClockOut : std_logic := '0';
	signal out1 : std_logic := '0';
	signal out2 : std_logic := '0';
	signal clockOut : std_logic := '0';

	-- Clock period definitions
	constant clockPeriod : time := 1us;

	-- controls values
	signal div1LowCountReg: std_logic_vector(4 downto 0) := (others => '0');
	signal div1HighEndReg: std_logic_vector(4 downto 0) := (others => '0');
	signal div2HighStartReg: std_logic_vector(4 downto 0) := (others => '0');
	signal div2HighCountReg: std_logic_vector(4 downto 0) := (others => '0');
	signal delayDiv2Reg: std_logic := '0';
	signal useDataInputReg: std_logic := '0';
	signal prescaleCounterReg: std_logic_vector(5 downto 0) := (others => '0');
	signal prescaleBypassReg: std_logic := '0';

	begin

	-- Instantiate the Unit Under Test (UUT)
	uut: main port map (
		clockIn => clockIn,
		data => data,
		controlData => controlData,
		controlClock => controlClock,
		outEnable => outEnable,
		rawClockOut => rawClockOut,
		out1 => out1,
		out2 => out2,
		clockOut => clockOut
	);

	-- Clock process definitions
	clockProcess: process
	begin
		clockIn <= '0';
		wait for clockPeriod/2;
		clockIn <= '1';
		wait for clockPeriod/2;
	end process;

	-- Stimulus process
	stim_proc: process
		procedure sendControl is
			variable word: std_logic_vector(28 downto 0);
		begin
			outEnable <= '0';
			wait for clockPeriod;
			word := prescaleBypassReg & prescaleCounterReg & useDataInputReg & delayDiv2Reg & div2HighCountReg & div2HighStartReg & div1HighEndReg & div1LowCountReg;
			wait for clockPeriod;
			for i in 0 to 28 loop
				controlData <= word(i);
				controlClock <= '0';
				wait for clockPeriod;
				controlClock <= '1';
				wait for clockPeriod;
			end loop;
			controlClock <= '0';
			outEnable <= '1';
		end;
	begin		
		wait for clockPeriod*10;

		-- test divide by 1
		div1LowCountReg <= std_logic_vector(to_unsigned(1, 5));
		div1HighEndReg <= std_logic_vector(to_unsigned(1, 5));
		div2HighStartReg <= std_logic_vector(to_unsigned(1, 5));
		div2HighCountReg <= std_logic_vector(to_unsigned(0, 5));
		delayDiv2Reg <= '1';
		useDataInputReg <= '0';
		prescaleBypassReg <= '1';
		sendControl;
		wait for clockPeriod*20/2;

		-- test divide by 2
		div1LowCountReg <= std_logic_vector(to_unsigned(2, 5));
		div1HighEndReg <= std_logic_vector(to_unsigned(3, 5));
		div2HighStartReg <= std_logic_vector(to_unsigned(3, 5));
		div2HighCountReg <= std_logic_vector(to_unsigned(1, 5));
		delayDiv2Reg <= '0';
		useDataInputReg <= '0';
		prescaleBypassReg <= '1';
		sendControl;
		wait for clockPeriod*20/2;

		-- test divide by 3
		div1LowCountReg <= std_logic_vector(to_unsigned(3, 5));
		div1HighEndReg <= std_logic_vector(to_unsigned(5, 5));
		div2HighStartReg <= std_logic_vector(to_unsigned(4, 5));
		div2HighCountReg <= std_logic_vector(to_unsigned(2, 5));
		delayDiv2Reg <= '1';
		useDataInputReg <= '0';
		prescaleBypassReg <= '1';
		sendControl;
		wait for clockPeriod*30/2;

		-- test divide by 4
		div1LowCountReg <= std_logic_vector(to_unsigned(4, 5));
		div1HighEndReg <= std_logic_vector(to_unsigned(7, 5));
		div2HighStartReg <= std_logic_vector(to_unsigned(6, 5));
		div2HighCountReg <= std_logic_vector(to_unsigned(3, 5));
		delayDiv2Reg <= '0';
		useDataInputReg <= '0';
		prescaleBypassReg <= '1';
		sendControl;
		wait for clockPeriod*40/2;

		-- test divide by 5
		div1LowCountReg <= std_logic_vector(to_unsigned(5, 5));
		div1HighEndReg <= std_logic_vector(to_unsigned(9, 5));
		div2HighStartReg <= std_logic_vector(to_unsigned(7, 5));
		div2HighCountReg <= std_logic_vector(to_unsigned(4, 5));
		delayDiv2Reg <= '1';
		useDataInputReg <= '0';
		prescaleBypassReg <= '1';
		sendControl;
		wait for clockPeriod*50/2;

		-- test divide by 6
		div1LowCountReg <= std_logic_vector(to_unsigned(6, 5));
		div1HighEndReg <= std_logic_vector(to_unsigned(11, 5));
		div2HighStartReg <= std_logic_vector(to_unsigned(9, 5));
		div2HighCountReg <= std_logic_vector(to_unsigned(5, 5));
		delayDiv2Reg <= '0';
		useDataInputReg <= '0';
		prescaleBypassReg <= '1';
		sendControl;
		wait for clockPeriod*60/2;

		assert false report "No failure, simulation was successful." severity failure; 

	end process;

end;
